@@ -85,7 +85,7 @@
 /*
   Define declarations.
 */
-#define XmpNamespaceExtent  28
+#define XMPNamespaceExtent  28
 
 /*
   Const declarations.
@@ -127,6 +127,20 @@ static MagickBooleanType
 %
 */
 
+static inline MagickBooleanType HEICSkipImage(const ImageInfo *image_info,
+  Image *image)
+{
+  if (image_info->number_scenes == 0)
+    return(MagickFalse);
+  if (image->scene == 0)
+    return(MagickFalse);
+  if (image->scene < image_info->scene)
+    return(MagickTrue);
+  if (image->scene > image_info->scene+image_info->number_scenes-1)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
 static inline MagickBooleanType IsHEIFSuccess(Image *image,
   struct heif_error *error,ExceptionInfo *exception)
 {
@@ -218,32 +232,80 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       StringInfo
         *profile;
 
-      /*
-        Skip first 4 bytes, the offset to the TIFF header.
-      */
-      profile=BlobToStringInfo(exif_profile+4,(size_t) length-4);
-      if (profile != (StringInfo*) NULL)
+      unsigned int
+        offset;
+
+      offset=(unsigned int) (*exif_profile) << 24;
+      offset|=(unsigned int) (*(exif_profile+1)) << 16;
+      offset|=(unsigned int) (*(exif_profile+2)) << 8;
+      offset|=(unsigned int) *(exif_profile+3);
+      offset+=4;
+      if (offset < (length-4))
         {
-          (void) SetImageProfile(image,"exif",profile,exception);
-          profile=DestroyStringInfo(profile);
+          length-=offset;
+          profile=BlobToStringInfo(exif_profile+offset,(size_t) length);
+          if (profile != (StringInfo*) NULL)
+            {
+              (void) SetImageProfile(image,"exif",profile,exception);
+              profile=DestroyStringInfo(profile);
+            }
         }
     }
   exif_profile=(unsigned char *) RelinquishMagickMemory(exif_profile);
   return(MagickTrue);
 }
 
-static inline MagickBooleanType HEICSkipImage(const ImageInfo *image_info,
-  Image *image)
+static MagickBooleanType ReadHEICXMPProfile(Image *image,
+  struct heif_image_handle *image_handle,ExceptionInfo *exception)
 {
-  if (image_info->number_scenes == 0)
-    return(MagickFalse);
-  if (image->scene == 0)
-    return(MagickFalse);
-  if (image->scene < image_info->scene)
+  heif_item_id
+    id;
+
+  int
+    count;
+
+  size_t
+    length;
+
+  struct heif_error
+    error;
+
+  unsigned char
+    *xmp_profile;
+
+  /*
+    Read XMP profile.
+  */
+  count=heif_image_handle_get_list_of_metadata_block_IDs(image_handle,"mime",
+    &id,1);
+  if (count != 1)
     return(MagickTrue);
-  if (image->scene > image_info->scene+image_info->number_scenes-1)
+  length=heif_image_handle_get_metadata_size(image_handle,id);
+  if (length <= 8)
     return(MagickTrue);
-  return(MagickFalse);
+  if ((MagickSizeType) length > GetBlobSize(image))
+    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+      image->filename);
+  xmp_profile=(unsigned char *) AcquireQuantumMemory(1,length);
+  if (xmp_profile == (unsigned char *) NULL)
+    return(MagickFalse);
+  error=heif_image_handle_get_metadata(image_handle,id,xmp_profile);
+  if ((IsHEIFSuccess(image,&error,exception) != MagickFalse) &&
+      (length > (XMPNamespaceExtent+2)))
+    {
+      StringInfo
+        *profile;
+
+      length-=(XMPNamespaceExtent+1);
+      profile=BlobToStringInfo(xmp_profile+XMPNamespaceExtent+1,length);
+      if (profile != (StringInfo*) NULL)
+        {
+          (void) SetImageProfile(image,"xmp",profile,exception);
+          profile=DestroyStringInfo(profile);
+        }
+    }
+  xmp_profile=(unsigned char *) RelinquishMagickMemory(xmp_profile);
+  return(MagickTrue);
 }
 
 static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
@@ -299,6 +361,8 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
     return(MagickFalse);
   if (ReadHEICExifProfile(image,image_handle,exception) == MagickFalse)
     return(MagickFalse);
+  if (ReadHEICXMPProfile(image,image_handle,exception) == MagickFalse)
+    return(MagickFalse);
   if (image_info->ping != MagickFalse)
     return(MagickTrue);
   if (HEICSkipImage(image_info,image) != MagickFalse)
@@ -660,6 +724,11 @@ ModuleExport size_t RegisterHEICImage(void)
   MagickInfo
     *entry;
 
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+#if LIBHEIF_NUMERIC_VERSION >= 0x010e0000
+  heif_init(NULL);
+#endif
+#endif
   entry=AcquireMagickInfo("HEIC","HEIC","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
@@ -734,6 +803,11 @@ ModuleExport void UnregisterHEICImage(void)
 #endif
   (void) UnregisterMagickInfo("HEIC");
   (void) UnregisterMagickInfo("HEIF");
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+#if LIBHEIF_NUMERIC_VERSION >= 0x010e0000
+  heif_deinit();
+#endif
+#endif
 }
 
 /*
@@ -824,7 +898,7 @@ static void WriteProfile(struct heif_context *context,Image *image,
           {
             if (profile != (StringInfo *) NULL)
               ConcatenateStringInfo(xmp_profile,profile);
-            GetStringInfoDatum(xmp_profile)[XmpNamespaceExtent]='\0';
+            GetStringInfoDatum(xmp_profile)[XMPNamespaceExtent]='\0';
             for (i=0; i < (ssize_t) GetStringInfoLength(xmp_profile); i+=65533L)
             {
               length=MagickMin(GetStringInfoLength(xmp_profile)-i,65533L);
@@ -1244,12 +1318,9 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     */
     if (lossless != MagickFalse)
       error=heif_encoder_set_lossless(heif_encoder,1);
-    else
-      if (image_info->quality == UndefinedCompressionQuality)
-        error=heif_encoder_set_lossy_quality(heif_encoder,50);
-      else
-        error=heif_encoder_set_lossy_quality(heif_encoder,(int)
-          image_info->quality);
+    else if (image_info->quality != UndefinedCompressionQuality)
+      error=heif_encoder_set_lossy_quality(heif_encoder,(int)
+        image_info->quality);
     status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
