@@ -16,7 +16,7 @@
 %                               December 2020                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2020 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,6 +54,7 @@
 #include "MagickCore/monitor.h"
 #include "MagickCore/monitor-private.h"
 #include "MagickCore/option.h"
+#include "MagickCore/property.h"
 #include "MagickCore/resource_.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
@@ -166,12 +167,17 @@ static void *JXLAcquireMemory(void *opaque,size_t size)
   return(data);
 }
 
-static inline StorageType JXLDataTypeToStorageType(const JxlDataType data_type)
+static inline StorageType JXLDataTypeToStorageType(Image *image,
+  const JxlDataType data_type,ExceptionInfo *exception)
 {
   switch (data_type)
   {
     case JXL_TYPE_FLOAT:
       return FloatPixel;
+    case JXL_TYPE_FLOAT16:
+      (void) SetImageProperty(image,"quantum:format","floating-point",
+        exception);
+      return FloatPixel;
     case JXL_TYPE_UINT16:
       return ShortPixel;
     case JXL_TYPE_UINT8:
@@ -222,14 +228,31 @@ static inline void JXLSetMemoryManager(JxlMemoryManager *memory_manager,
   memory_manager->free=JXLRelinquishMemory;
 }
 
-static inline void JXLSetFormat(Image *image,JxlPixelFormat *pixel_format)
+static inline void JXLSetFormat(Image *image,JxlPixelFormat *pixel_format,
+  ExceptionInfo *exception)
 {
+  const char
+    *property;
+
   pixel_format->num_channels=(image->alpha_trait == BlendPixelTrait) ? 4U : 3U;
   if (IsGrayColorspace(image->colorspace) != MagickFalse)
     pixel_format->num_channels=(image->alpha_trait == BlendPixelTrait) ?
       2U : 1U;
   pixel_format->data_type=(image->depth > 16) ? JXL_TYPE_FLOAT :
     (image->depth > 8) ? JXL_TYPE_UINT16 : JXL_TYPE_UINT8;
+  property=GetImageProperty(image,"quantum:format",exception);
+  if (property != (char *) NULL)
+    {
+      QuantumFormatType format = (QuantumFormatType) ParseCommandOption(
+        MagickQuantumFormatOptions,MagickFalse,property);
+      if ((format == FloatingPointQuantumFormat) &&
+          (pixel_format->data_type == JXL_TYPE_UINT16))
+        {
+          pixel_format->data_type=JXL_TYPE_FLOAT16;
+          (void) SetImageProperty(image,"quantum:format","floating-point",
+            exception);
+        }
+    }
 }
 
 static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -248,7 +271,7 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     memory_manager;
 
   JxlPixelFormat
-    pixel_format;
+    pixel_format = { 0 };
 
   MagickBooleanType
     status;
@@ -259,9 +282,13 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     input_size;
 
+  StringInfo
+    *exif_profile = (StringInfo *) NULL,
+    *xmp_profile = (StringInfo *) NULL;
+
   unsigned char
     *pixels,
-    *output_buffer;
+    *output_buffer = (unsigned char *) NULL;
 
   void
     *runner;
@@ -271,11 +298,11 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info, exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -303,9 +330,10 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       JxlDecoderDestroy(jxl_info);
       ThrowReaderException(CoderError,"MemoryAllocationFailed");
     }
-  events_wanted=JXL_DEC_BASIC_INFO;
+  events_wanted=(JxlDecoderStatus) (JXL_DEC_BASIC_INFO | JXL_DEC_BOX);
   if (image_info->ping == MagickFalse)
-    events_wanted|=JXL_DEC_FULL_IMAGE | JXL_DEC_COLOR_ENCODING;
+    events_wanted=(JxlDecoderStatus) (events_wanted | JXL_DEC_FULL_IMAGE |
+      JXL_DEC_COLOR_ENCODING);
   if (JxlDecoderSubscribeEvents(jxl_info,events_wanted) != JXL_DEC_SUCCESS)
     {
       JxlThreadParallelRunnerDestroy(runner);
@@ -313,15 +341,13 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CoderError,"UnableToReadImageData");
     }
   input_size=MagickMaxBufferExtent;
-  pixels=AcquireQuantumMemory(input_size,sizeof(*pixels));
+  pixels=(unsigned char *) AcquireQuantumMemory(input_size,sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     {
       JxlThreadParallelRunnerDestroy(runner);
       JxlDecoderDestroy(jxl_info);
       ThrowReaderException(CoderError,"MemoryAllocationFailed");
     }
-  output_buffer=(unsigned char *) NULL;
-  (void) memset(&pixel_format,0,sizeof(pixel_format));
   jxl_status=JXL_DEC_NEED_MORE_INPUT;
   while ((jxl_status != JXL_DEC_SUCCESS) && (jxl_status != JXL_DEC_ERROR))
   {
@@ -331,17 +357,18 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       case JXL_DEC_NEED_MORE_INPUT:
       {
         size_t
-          remaining = JxlDecoderReleaseInput(jxl_info);
+          remaining;
 
         ssize_t
           count;
 
+        remaining=JxlDecoderReleaseInput(jxl_info);
         if (remaining > 0)
           memmove(pixels,pixels+input_size-remaining,remaining);
         count=ReadBlob(image,input_size-remaining,pixels+remaining);
         if (count <= 0)
           {
-            jxl_status=JXL_DEC_ERROR;
+            JxlDecoderCloseInput(jxl_info);
             break;
           }
         jxl_status=JxlDecoderSetInput(jxl_info,(const uint8_t *) pixels,
@@ -389,20 +416,24 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         StringInfo
           *profile;
 
-        JXLSetFormat(image,&pixel_format);
+        JXLSetFormat(image,&pixel_format,exception);
         (void) memset(&color_encoding,0,sizeof(color_encoding));
         jxl_status=JxlDecoderGetColorAsEncodedProfile(jxl_info,&pixel_format,
           JXL_COLOR_PROFILE_TARGET_DATA,&color_encoding);
-        if (jxl_status != JXL_DEC_SUCCESS)
-          break;
-        if (color_encoding.transfer_function == JXL_TRANSFER_FUNCTION_LINEAR)
-          image->colorspace=RGBColorspace;
-        if (color_encoding.color_space == JXL_COLOR_SPACE_GRAY)
+        if (jxl_status == JXL_DEC_SUCCESS)
           {
-            image->colorspace=GRAYColorspace;
             if (color_encoding.transfer_function == JXL_TRANSFER_FUNCTION_LINEAR)
-              image->colorspace=LinearGRAYColorspace;
+              image->colorspace=RGBColorspace;
+            if (color_encoding.color_space == JXL_COLOR_SPACE_GRAY)
+              {
+                image->colorspace=GRAYColorspace;
+                if (color_encoding.transfer_function == JXL_TRANSFER_FUNCTION_LINEAR)
+                  image->colorspace=LinearGRAYColorspace;
+              }
           }
+        else
+          if (jxl_status != JXL_DEC_ERROR)
+            break;
         jxl_status=JxlDecoderGetICCProfileSize(jxl_info,&pixel_format,
           JXL_COLOR_PROFILE_TARGET_ORIGINAL,&profile_size);
         if (jxl_status != JXL_DEC_SUCCESS)
@@ -425,11 +456,12 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=SetImageExtent(image,image->columns,image->rows,exception);
         if (status == MagickFalse)
           break;
-        JXLSetFormat(image,&pixel_format);
+        JXLSetFormat(image,&pixel_format,exception);
         jxl_status=JxlDecoderImageOutBufferSize(jxl_info,&pixel_format,&extent);
         if (jxl_status != JXL_DEC_SUCCESS)
           break;
-        output_buffer=AcquireQuantumMemory(extent,sizeof(*output_buffer));
+        output_buffer=(unsigned char *) AcquireQuantumMemory(extent,
+          sizeof(*output_buffer));
         if (output_buffer == (unsigned char *) NULL)
           {
             (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
@@ -440,9 +472,13 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
           output_buffer,extent);
         if (jxl_status == JXL_DEC_SUCCESS)
           jxl_status=JXL_DEC_NEED_IMAGE_OUT_BUFFER;
+        break;
       }
       case JXL_DEC_FULL_IMAGE:
       {
+        const char
+          *map = "RGB";
+
         StorageType
           type;
 
@@ -452,28 +488,100 @@ static Image *ReadJXLImage(const ImageInfo *image_info,ExceptionInfo *exception)
               CorruptImageError,"UnableToReadImageData","`%s'",image->filename);
             break;
           }
-        type=JXLDataTypeToStorageType(pixel_format.data_type);
+        type=JXLDataTypeToStorageType(image,pixel_format.data_type,exception);
         if (type == UndefinedPixel)
           {
             (void) ThrowMagickException(exception,GetMagickModule(),
               CorruptImageError,"Unsupported data type","`%s'",image->filename);
             break;
           }
-        status=ImportImagePixels(image,0,0,image->columns,image->rows,
-          image->alpha_trait == BlendPixelTrait ? "RGBA" : "RGB",type,
-          output_buffer,exception);
+        if (image->alpha_trait == BlendPixelTrait)
+          map="RGBA";
+        if (IsGrayColorspace(image->colorspace) != MagickFalse)
+          {
+            map="I";
+            if (image->alpha_trait == BlendPixelTrait)
+              map="IA";
+          }
+        status=ImportImagePixels(image,0,0,image->columns,image->rows,map,
+          type,output_buffer,exception);
         if (status == MagickFalse)
           jxl_status=JXL_DEC_ERROR;
         break;
       }
+      case JXL_DEC_BOX:
+      {
+        JxlBoxType
+          type;
+
+        uint64_t
+          size;
+
+        (void) JxlDecoderReleaseBoxBuffer(jxl_info);
+        jxl_status=JxlDecoderGetBoxType(jxl_info,type,JXL_FALSE);
+        if (jxl_status != JXL_DEC_SUCCESS)
+          break;
+        jxl_status=JxlDecoderGetBoxSizeRaw(jxl_info,&size);
+        if ((jxl_status != JXL_DEC_SUCCESS) || (size < 4))
+          break;
+        if (LocaleNCompare(type,"Exif",sizeof(type)) == 0)
+          {
+            unsigned char
+              *p;
+
+            unsigned int
+              offset;
+
+            /*
+              Prepend a 4-byte TIFF header offset.
+            */
+            exif_profile=AcquireStringInfo((size_t) size);
+            p=GetStringInfoDatum(exif_profile);
+            jxl_status=JxlDecoderSetBoxBuffer(jxl_info,p,size);
+            offset=(unsigned int) (*p) << 24;
+            offset|=(unsigned int) (*(p+1)) << 16;
+            offset|=(unsigned int) (*(p+2)) << 8;
+            offset|=(unsigned int) *(p+3);
+            offset+=4;
+            if (offset < (size-8))
+              {
+                (void) DestroyStringInfo(SplitStringInfo(exif_profile,offset));
+                SetStringInfoLength(exif_profile,size-offset-8);
+              }
+          }
+        if (LocaleNCompare(type,"xml ",sizeof(type)) == 0)
+          {
+            xmp_profile=AcquireStringInfo((size_t) size);
+            jxl_status=JxlDecoderSetBoxBuffer(jxl_info,
+              GetStringInfoDatum(xmp_profile),size);
+          }
+        if (jxl_status == JXL_DEC_SUCCESS)
+          jxl_status=JXL_DEC_BOX;
+        break;
+      }
       case JXL_DEC_SUCCESS:
       case JXL_DEC_ERROR:
         break;
       default:
+      {
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          CorruptImageError,"Unsupported status type","`%d'",jxl_status);
         jxl_status=JXL_DEC_ERROR;
         break;
+      }
     }
   }
+  (void) JxlDecoderReleaseBoxBuffer(jxl_info);
+  if (exif_profile != (StringInfo *) NULL)
+    {
+      (void) SetImageProfile(image,"exif",exif_profile,exception);
+      exif_profile=DestroyStringInfo(exif_profile);
+    }
+  if (xmp_profile != (StringInfo *) NULL)
+    {
+      (void) SetImageProfile(image,"xmp",xmp_profile,exception);
+      xmp_profile=DestroyStringInfo(xmp_profile);
+    }
   output_buffer=(unsigned char *) RelinquishMagickMemory(output_buffer);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   JxlThreadParallelRunnerDestroy(runner);
@@ -609,20 +717,33 @@ static JxlEncoderStatus JXLWriteMetadata(const Image *image,
   return(jxl_status);
 }
 
+static inline float JXLGetDistance(const ImageInfo *image_info)
+{
+  if (image_info->quality == 0)
+    return(1.0f);
+  if (image_info->quality >= 30)
+    return(0.1f+(float) (100-MagickMin(100,image_info->quality))*0.09f);
+  return(6.24f+(float) pow(2.5f,(30.0-image_info->quality)/5.0f)/6.25f);
+}
+
 static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
   const char
     *option;
 
+  const StringInfo
+    *exif_profile = (StringInfo *) NULL,
+    *xmp_profile = (StringInfo *) NULL;
+
   JxlBasicInfo
     basic_info;
 
   JxlEncoder
     *jxl_info;
 
-  JxlEncoderOptions
-    *jxl_options;
+  JxlEncoderFrameSettings
+    *frame_settings;
 
   JxlEncoderStatus
     jxl_status;
@@ -658,10 +779,10 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
@@ -685,7 +806,7 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
       return(MagickFalse);
     }
   (void) memset(&pixel_format,0,sizeof(pixel_format));
-  JXLSetFormat(image,&pixel_format);
+  JXLSetFormat(image,&pixel_format,exception);
   (void) memset(&basic_info,0,sizeof(basic_info));
   JxlEncoderInitBasicInfo(&basic_info);
   basic_info.xsize=(uint32_t) image->columns;
@@ -699,8 +820,20 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
         basic_info.bits_per_sample=32;
         basic_info.exponent_bits_per_sample=8;
       }
+    else
+      if (pixel_format.data_type == JXL_TYPE_FLOAT16)
+        {
+          basic_info.bits_per_sample=16;
+          basic_info.exponent_bits_per_sample=8;
+        }
+  if (IsGrayColorspace(image->colorspace) != MagickFalse)
+    basic_info.num_color_channels=1;
   if (image->alpha_trait == BlendPixelTrait)
-    basic_info.alpha_bits=basic_info.bits_per_sample;
+    {
+      basic_info.alpha_bits=basic_info.bits_per_sample;
+      basic_info.alpha_exponent_bits=basic_info.exponent_bits_per_sample;
+      basic_info.num_extra_channels=1;
+    }
   if (image->quality == 100)
     basic_info.uses_original_profile=JXL_TRUE;
   jxl_status=JxlEncoderSetBasicInfo(jxl_info,&basic_info);
@@ -710,34 +843,54 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
       JxlEncoderDestroy(jxl_info);
       ThrowWriterException(CoderError,"UnableToWriteImageData");
     }
-  jxl_options=JxlEncoderOptionsCreate(jxl_info,(JxlEncoderOptions *) NULL);
-  if (jxl_options == (JxlEncoderOptions *) NULL)
+  frame_settings=JxlEncoderFrameSettingsCreate(jxl_info,
+    (JxlEncoderFrameSettings *) NULL);
+  if (frame_settings == (JxlEncoderFrameSettings *) NULL)
     {
       JxlThreadParallelRunnerDestroy(runner);
       JxlEncoderDestroy(jxl_info);
       ThrowWriterException(CoderError,"MemoryAllocationFailed");
     }
   if (image->quality == 100)
-    (void) JxlEncoderOptionsSetLossless(jxl_options,JXL_TRUE);
+    (void) JxlEncoderSetFrameLossless(frame_settings,JXL_TRUE);
   else
-    {
-      float
-        distance;
-
-      distance=(image_info->quality >= 30) ? 0.1f+(float) (100-MagickMin(100,
-        image_info->quality))*0.09f : 6.4f+(float) pow(2.5f,(30.0-
-        image_info->quality)/5.0f)/6.25f;
-      (void) JxlEncoderOptionsSetDistance(jxl_options,distance);
-    }
+    (void) JxlEncoderSetFrameDistance(frame_settings,
+      JXLGetDistance(image_info));
   option=GetImageOption(image_info,"jxl:effort");
   if (option != (const char *) NULL)
-    (void) JxlEncoderOptionsSetEffort(jxl_options,StringToInteger(option));
+    (void) JxlEncoderFrameSettingsSetOption(frame_settings,
+      JXL_ENC_FRAME_SETTING_EFFORT,StringToInteger(option));
   option=GetImageOption(image_info,"jxl:decoding-speed");
   if (option != (const char *) NULL)
-    (void) JxlEncoderOptionsSetDecodingSpeed(jxl_options,
-      StringToInteger(option));
+    (void) JxlEncoderFrameSettingsSetOption(frame_settings,
+      JXL_ENC_FRAME_SETTING_DECODING_SPEED,StringToInteger(option));
+  exif_profile=GetImageProfile(image,"exif");
+  xmp_profile=GetImageProfile(image,"xmp");
+  if ((exif_profile != (StringInfo *) NULL) ||
+      (xmp_profile != (StringInfo *) NULL))
+    {
+      /*
+        Add metadata boxes.
+      */
+      (void) JxlEncoderUseBoxes(jxl_info);
+      if (exif_profile != (StringInfo *) NULL)
+        {
+          /*
+            Prepend a 4-byte TIFF header offset.
+          */
+          StringInfo *profile = AcquireStringInfo(sizeof(unsigned int));
+          (void) memset(GetStringInfoDatum(profile),0,sizeof(unsigned int));
+          ConcatenateStringInfo(profile,exif_profile);
+          (void) JxlEncoderAddBox(jxl_info,"Exif",GetStringInfoDatum(profile),
+            GetStringInfoLength(profile),0);
+          profile=DestroyStringInfo(profile);
+        }
+      if (xmp_profile != (StringInfo *) NULL)
+        (void) JxlEncoderAddBox(jxl_info,"xml ",GetStringInfoDatum(xmp_profile),
+          GetStringInfoLength(xmp_profile),0);
+      (void) JxlEncoderCloseBoxes(jxl_info);
+    }
   jxl_status=JXLWriteMetadata(image,jxl_info);
-  jxl_status=JXL_ENC_SUCCESS;
   if (jxl_status != JXL_ENC_SUCCESS)
     {
       JxlThreadParallelRunnerDestroy(runner);
@@ -766,11 +919,13 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
   if (IsGrayColorspace(image->colorspace) != MagickFalse)
     status=ExportImagePixels(image,0,0,image->columns,image->rows,
       image->alpha_trait == BlendPixelTrait ? "IA" : "I",
-      JXLDataTypeToStorageType(pixel_format.data_type),pixels,exception);
+      JXLDataTypeToStorageType(image,pixel_format.data_type,exception),pixels,
+      exception);
   else
     status=ExportImagePixels(image,0,0,image->columns,image->rows,
       image->alpha_trait == BlendPixelTrait ? "RGBA" : "RGB",
-      JXLDataTypeToStorageType(pixel_format.data_type),pixels,exception);
+      JXLDataTypeToStorageType(image,pixel_format.data_type,exception),pixels,
+      exception);
   if (status == MagickFalse)
     {
       pixel_info=RelinquishVirtualMemory(pixel_info);
@@ -778,7 +933,7 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
       JxlEncoderDestroy(jxl_info);
       ThrowWriterException(CoderError,"MemoryAllocationFailed");
     }
-  jxl_status=JxlEncoderAddImageFrame(jxl_options,&pixel_format,pixels,
+  jxl_status=JxlEncoderAddImageFrame(frame_settings,&pixel_format,pixels,
     bytes_per_row*image->rows);
   if (jxl_status == JXL_ENC_SUCCESS)
     {
@@ -786,8 +941,8 @@ static MagickBooleanType WriteJXLImage(const ImageInfo *image_info,Image *image,
         *output_buffer;
 
       JxlEncoderCloseInput(jxl_info);
-      output_buffer=AcquireQuantumMemory(MagickMaxBufferExtent,
-        sizeof(*output_buffer));
+      output_buffer=(unsigned char *) AcquireQuantumMemory(
+        MagickMaxBufferExtent,sizeof(*output_buffer));
       if (output_buffer == (unsigned char *) NULL)
         {
           pixel_info=RelinquishVirtualMemory(pixel_info);
